#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Dieses Python-Skript erstellt eine M3U-Playlist aus den Audio-Dateien in dem Verzeichnis, in dem das Skript ausgeführt wird. Es unterstützt .mp3- und .mp4-Dateien und ist darauf ausgelegt, Metadaten von .mp3-Dateien zu extrahieren, um diese in die Playlist zu integrieren.

Erstellt mit Grok und ChatGPT unter Anleitung
von Volker Lange-Janson SM5ZBS am 17. Februar 2025
Jeder darf damit machen was er will.
"""


import os
try:
    from mutagen.mp3 import MP3
    mutagen_available = True
except ImportError:
    mutagen_available = False

# Definiere die Dateiendungen, die in die Playlist aufgenommen werden sollen
erlaubte_endungen = ('.mp4', '.mp3')

# Ermittle das Verzeichnis, in dem sich das Skript befindet
skript_verzeichnis = os.path.dirname(os.path.abspath(__file__))

# Erstelle den Namen der m3u-Datei
m3u_datei = os.path.join(skript_verzeichnis, 'playlist.m3u')

def get_mp3_info(datei):
    """Versucht, MP3-Metadaten zu extrahieren, wenn mutagen verfügbar ist."""
    if not mutagen_available:
        return "Unknown Artist - Unknown Title", None, 0
    try:
        audio = MP3(os.path.join(skript_verzeichnis, datei))
        title = audio.get('TIT2', ['Unknown Title'])[0]
        artist = audio.get('TPE1', ['Unknown Artist'])[0]
        album = audio.get('TALB', ['Unknown Album'])[0]
        track = audio.get('TRCK', ['0'])[0].split('/')[0]  # Nur die erste Zahl der Tracknummer nehmen
        duration = int(audio.info.length) if audio.info else 0
        return f"{artist} - {title} ({album})", int(track) if track.isdigit() else None, duration
    except Exception:
        return "Unknown Artist - Unknown Title", None, 0

# Dateien filtern und Tracknummern auslesen
def datei_sortierschluessel(datei):
    if datei.lower().endswith('.mp3'):
        _, tracknummer, _ = get_mp3_info(datei)
        return (tracknummer if tracknummer is not None else float('inf'), datei.lower())
    return (float('inf'), datei.lower())

dateien = sorted((f for f in os.listdir(skript_verzeichnis) if f.lower().endswith(erlaubte_endungen)), key=datei_sortierschluessel)

# M3U-Datei schreiben
with open(m3u_datei, 'w', encoding='utf-8') as playlist:
    playlist.write("#EXTM3U\n")
    for datei in dateien:
        info, _, duration = get_mp3_info(datei) if datei.lower().endswith('.mp3') else (datei, None, 0)
        playlist.write(f"#EXTINF:{duration},{info}\n{datei}\n")
        
        # Falls MP3, Metadaten in der Konsole ausgeben
        if datei.lower().endswith('.mp3'):
            print(f"{datei}: {info}")

print(f"Playlist erfolgreich erstellt: {m3u_datei}")

"""
Dieses Python-Skript erstellt eine M3U-Playlist aus den Audio-Dateien in dem Verzeichnis, in dem das Skript ausgeführt wird. Es unterstützt .mp3- und .mp4-Dateien und ist darauf ausgelegt, Metadaten von .mp3-Dateien zu extrahieren, um diese in die Playlist zu integrieren.

Schritt-für-Schritt-Erklärung:

    Import und Fehlerbehandlung:
        Das Skript versucht, das Modul mutagen.mp3 zu importieren, um MP3-Metadaten wie Titel, Künstler und Album auslesen zu können. Wenn mutagen nicht installiert ist, wird eine Alternative verwendet, die allgemeine Informationen wie „Unbekannter Titel“ und „Unbekannter Künstler“ zurückgibt.

    Festlegung der erlaubten Dateiendungen:
        Es werden nur .mp4 und .mp3-Dateien in die Playlist aufgenommen.

    Verzeichnis und M3U-Datei erstellen:
        Das Skript ermittelt das Verzeichnis, in dem es ausgeführt wird, und erstellt die Playlist-Datei playlist.m3u im selben Verzeichnis.

    MP3-Metadaten extrahieren (get_mp3_info):
        Diese Funktion liest Metadaten von .mp3-Dateien, darunter Titel, Künstler, Album und Tracknummer. Die Dauer des Tracks wird ebenfalls extrahiert. Falls die Datei keine Metadaten hat oder ein Fehler auftritt, wird „Unbekannter Titel“ und „Unbekannter Künstler“ zurückgegeben.

    Dateien sortieren (datei_sortierschluessel):
        Die Dateien im Verzeichnis werden nach Dateiendung gefiltert und nach Tracknummer sortiert, falls vorhanden. Die Sortierung ermöglicht es, die Tracks in der richtigen Reihenfolge anzuzeigen (falls Tracknummern vorhanden sind). Dateien ohne Tracknummern oder mit einer ungültigen Tracknummer werden ans Ende der Liste gestellt.

    M3U-Datei schreiben:
        Das Skript öffnet die Datei playlist.m3u und schreibt die M3U-Playlist im Standardformat (#EXTM3U). Für jede Datei werden die Metadaten (falls vorhanden) in das Playlist-Format (#EXTINF:duration,title) und der Dateiname geschrieben.

    Metadaten in der Konsole ausgeben:
        Für .mp3-Dateien werden die Metadaten (wie Künstler, Titel, Album) in der Konsole angezeigt.

    Erfolgsmeldung:
        Am Ende des Skripts wird eine Bestätigung ausgegeben, dass die Playlist erfolgreich erstellt wurde.

Fazit: Das Skript bietet eine effiziente Möglichkeit, eine M3U-Playlist zu erstellen, die .mp3- und .mp4-Dateien aus einem Verzeichnis enthält, wobei .mp3-Dateien mit Metadaten angereichert werden. Es ist nützlich, wenn man eine Playlist mit den richtigen Informationen und einer korrekten Reihenfolge von Musikstücken erstellen möchte, insbesondere für den Einsatz in Mediaplayern, die M3U unterstützen.
"""

