import os
import time
from http.server import HTTPServer, BaseHTTPRequestHandler
import socketserver
import errno
import threading
import tkinter as tk
from tkinter import ttk, messagebox
import configparser

# Farbschema (sanfte Graublau-Töne)
COLORS = {
    "bg_dark": "#2E3B4E",
    "bg_main": "#3D4F65",
    "bg_light": "#4A5D75",
    "text": "#E0E7EF",
    "accent": "#6B8CAE",
    "button": "#5A7A9E"
}

# Konfigurationsdatei für Pfade
CONFIG_FILE = "webradio_paths.ini"

# Maximale Anzahl der Clients (für zukünftige Erweiterungen, aktuell nicht begrenzt)
MAX_CLIENTS = 5

class RadioStreamHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == "/stream":
            self.send_response(200)
            self.send_header("Content-type", "audio/mpeg")
            self.send_header("Transfer-Encoding", "chunked")
            self.send_header("Cache-Control", "no-cache")
            self.send_header("Connection", "keep-alive")
            self.send_header("icy-name", "Mein LAN-Radio")
            self.end_headers()

            playlist = self.server.get_playlist()
            if not playlist:
                self.wfile.write(b"0\r\n\r\n")
                return

            while True:
                for mp3_file in playlist:
                    try:
                        with open(mp3_file, "rb") as f:
                            audio_data = f.read()
                            chunk_size = len(audio_data)
                            self.wfile.write(f"{chunk_size:x}\r\n".encode())
                            self.wfile.write(audio_data)
                            self.wfile.write(b"\r\n")
                            self.wfile.flush()
                            print(f"Spiele für Client {self.client_address}: {os.path.basename(mp3_file)}")
                    except IOError as e:
                        if e.errno == errno.EPIPE:
                            print(f"Client {self.client_address} getrennt bei {mp3_file}, fahre fort...")
                            return  # Client-Verbindung endet, aber andere können weiterlaufen
                        else:
                            print(f"Fehler bei {mp3_file} für Client {self.client_address}: {e}")
                            return
                    except Exception as e:
                        print(f"Unbekannter Fehler bei {mp3_file} für Client {self.client_address}: {e}")
                        return
                time.sleep(0.1)  # Kurze Pause zwischen Songs für Synchronisation

        else:
            self.send_error(404, "Nur /stream verfügbar")

class ThreadingRadioServer(socketserver.ThreadingMixIn, HTTPServer):
    def __init__(self, server_address, handler_class, music_dir=None, m3u_file=None):
        super().__init__(server_address, handler_class)
        self.music_dir = music_dir
        self.m3u_file = m3u_file
        self.playlist = self.load_playlist()

    def load_playlist(self):
        if self.m3u_file and os.path.exists(self.m3u_file):
            with open(self.m3u_file, "r") as f:
                playlist = []
                for line in f:
                    line = line.strip()
                    if line and line.endswith(".mp3"):
                        # Absoluter Pfad oder relativ zum M3U-Ordner
                        if os.path.isabs(line):
                            mp3_path = line
                        else:
                            mp3_path = os.path.join(os.path.dirname(self.m3u_file), line)
                        if os.path.exists(mp3_path):
                            playlist.append(mp3_path)
                        else:
                            print(f"Datei nicht gefunden: {mp3_path}")
                return playlist
        elif self.music_dir:
            return [os.path.join(self.music_dir, f) for f in os.listdir(self.music_dir) if f.endswith(".mp3")]
        return []

    def get_playlist(self):
        return self.playlist

class WebradioApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Webradio Streaming Server")
        self.root.geometry("600x500")
        self.root.configure(bg=COLORS["bg_main"])

        self.server = None
        self.server_thread = None
        self.paths = self.load_paths()
        
        self.create_styles()
        self.create_widgets()

    def load_paths(self):
        """Lade Pfade aus der INI-Datei"""
        config = configparser.ConfigParser()
        if not os.path.exists(CONFIG_FILE):
            # Standard-Pfade, falls die INI-Datei nicht existiert
            config['Paths'] = {
                'path1': '/home/volker/Schreibtisch/MP3 Mix Smooth Jazz',
                'path2': '/home/volker/Musik'
            }
            with open(CONFIG_FILE, 'w') as configfile:
                config.write(configfile)
        config.read(CONFIG_FILE)
        return list(config['Paths'].values())

    def create_styles(self):
        """Definiere die Stile für ttk-Widgets"""
        self.style = ttk.Style()
        self.style.configure("TFrame", background=COLORS["bg_main"])
        self.style.configure("TLabel", 
                            background=COLORS["bg_main"], 
                            foreground=COLORS["text"],
                            font=("Arial", 10))
        self.style.configure("TButton", 
                            background=COLORS["button"],
                            foreground=COLORS["text"],
                            font=("Arial", 10, "bold"))
        self.style.configure("TListbox", 
                            background=COLORS["bg_light"],
                            foreground=COLORS["text"])
        self.style.map("TButton",
                      background=[("active", COLORS["accent"])])
        self.style.configure("Header.TLabel",
                            font=("Arial", 14, "bold"),
                            padding=10)
        self.style.configure("Section.TLabel",
                            font=("Arial", 12, "bold"),
                            padding=(0, 10, 0, 5))

    def create_widgets(self):
        """Erstelle die GUI-Elemente"""
        main_frame = ttk.Frame(self.root)
        main_frame.pack(fill=tk.BOTH, expand=True, padx=20, pady=20)

        # Titel
        title_label = ttk.Label(main_frame, text="Webradio Streaming Server", 
                               style="Header.TLabel")
        title_label.pack(fill=tk.X)

        # Auswahlbereich
        selection_section = ttk.Label(main_frame, text="Ordner oder M3U auswählen:",
                                     style="Section.TLabel")
        selection_section.pack(fill=tk.X)

        self.listbox = tk.Listbox(main_frame, 
                                 bg=COLORS["bg_light"],
                                 fg=COLORS["text"],
                                 selectbackground=COLORS["accent"],
                                 relief="flat",
                                 height=10)
        self.listbox.pack(fill=tk.BOTH, expand=True, pady=5)

        # Fülle die Listbox mit Pfaden
        for path in self.paths:
            self.listbox.insert(tk.END, path)

        # Aktionsbereich
        actions_frame = ttk.Frame(main_frame)
        actions_frame.pack(fill=tk.X, pady=10)

        start_button = ttk.Button(actions_frame, 
                                 text="Stream starten", 
                                 command=self.start_stream)
        start_button.pack(side=tk.LEFT, padx=5)

        stop_button = ttk.Button(actions_frame, 
                                text="Stream stoppen", 
                                command=self.stop_stream)
        stop_button.pack(side=tk.LEFT, padx=5)

        # Status
        self.status_var = tk.StringVar()
        self.status_var.set("Bereit")

        status_frame = ttk.Frame(main_frame)
        status_frame.pack(fill=tk.X, pady=10)

        status_label = ttk.Label(status_frame, text="Status:")
        status_label.pack(side=tk.LEFT, padx=5)

        status_text = ttk.Label(status_frame, textvariable=self.status_var)
        status_text.pack(side=tk.LEFT, padx=5)

    def start_stream(self):
        """Starte den Streaming-Server"""
        if self.server:
            messagebox.showwarning("Warnung", "Der Server läuft bereits!")
            return

        selection = self.listbox.curselection()
        if not selection:
            messagebox.showwarning("Warnung", "Bitte wählen Sie einen Ordner oder eine M3U-Datei aus!")
            return

        selected_path = self.listbox.get(selection[0])
        music_dir = None
        m3u_file = None

        if selected_path.endswith(".m3u"):
            m3u_file = selected_path
        else:
            music_dir = selected_path

        self.status_var.set("Starte Server...")
        self.root.update_idletasks()

        try:
            server_address = ("0.0.0.0", 8000)
            self.server = ThreadingRadioServer(server_address, RadioStreamHandler, music_dir, m3u_file)
            local_ip = self.get_local_ip()
            self.status_var.set(f"LAN-Radio läuft auf http://{local_ip}:8000/stream (bis zu {MAX_CLIENTS} Clients)")
            self.server_thread = threading.Thread(target=self.server.serve_forever)
            self.server_thread.daemon = True
            self.server_thread.start()
        except Exception as e:
            self.status_var.set("Fehler beim Starten des Servers")
            messagebox.showerror("Fehler", f"Fehler beim Starten des Servers: {e}")
            self.server = None

    def stop_stream(self):
        """Stoppe den Streaming-Server"""
        if not self.server:
            messagebox.showwarning("Warnung", "Kein Server läuft!")
            return

        self.status_var.set("Stoppe Server...")
        self.root.update_idletasks()

        try:
            self.server.shutdown()
            self.server.server_close()
            self.server = None
            self.server_thread = None
            self.status_var.set("Server gestoppt")
        except Exception as e:
            self.status_var.set("Fehler beim Stoppen")
            messagebox.showerror("Fehler", f"Fehler beim Stoppen des Servers: {e}")

    def get_local_ip(self):
        import socket
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(("8.8.8.8", 80))
            ip = s.getsockname()[0]
        except Exception:
            ip = "localhost"
        finally:
            s.close()
        return ip

if __name__ == "__main__":
    root = tk.Tk()
    app = WebradioApp(root)
    root.mainloop()
